//
//  main.c
//
//  DeskClock 0.2
//  Copyright(C)2011 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		This is a clock for the desk.
//		4-digit LED display, USB power, USB control.
//
//	Usage
//		Connect to USB power adapter or PC.
//		Input now time by push button.
//		The command control from the PC COM port(virtual serial port)
//
//  Device: PIC18F26J50
//
//  Pin assign:
//		p1:  MCLR							Pull-up
//		p2:  RA0/AN0/C1INA/ULPWU/RP0		Push switch <mode button>
//		p3:  RA1/AN1/C2INA/RP1				Push switch <up button>
//		p4:  RA2/AN2/VREF-/CVREF/C2INB		Push switch <down button>
//		p5:  RA3/AN3/VREF+/C1INB			External input
//		p6:  VDDCORE/VCAP(2)				10uF
//		p7:  RA5/AN4/SS1/HLVDIN/RCV/RP2		Buzzer
//		p8:  VSS							GND
//		p9:  OSC1/CLKI/RA7					NC
//		p10: OSC2/CLKO/RA6					7-Seg DIG1
//		p11: RC0/T1OSO/T1CKI/RP11			Xtal 32.768kHz
//		p12: RC1/T1OSI/UOE/RP12				Xtal 32.768kHz
//		p13: RC2/AN11/CTPLS/RP13			7-Seg DIG2
//		p14: VUSB							3.3V
//		p15: RC4/D-/VM						USB
//		p16: RC5/D+/VP						USB
//		p17: RC6/TX1/CK1/RP17				7-Seg DIG3
//		p18: RC7/RX1/DT1/SDO1/RP18			7-Seg DIG4
//		p19: VSS							GND
//		p20: VDD							3.3V
//		p21: RB0/AN12/INT0/RP3				7-Seg a
//		p22: RB1/AN10/RTCC/RP4				7-Seg b
//		p23: RB2/AN8/CTEDG1/VMO/REFO/RP5	7-Seg c
//		p24: RB3/AN9/CTEDG2/VPO/RP6			7-Seg d
//		p25: RB4/KBI0/SCK1/SCL1/RP7			7-Seg e
//		p26: RB5/KBI1/SDI1/SDA1/RP8			7-Seg f
//		p27: RB6/KBI2/PGC/RP9				7-Seg g
//		p28: RB7/KBI3/PGD/RP10				7-Seg dp
//
//	Clock:	Fosc=48MHz
//
//	Internal peripheral module:
//		BOR:	-
//		Timer0:	-
//		Timer1:	LED refresh interval
//		Timer2:	-
//		Timer3:	-
//		Comp:	-
//		ECCP1:	-
//		USB:	Serial emuration(high speed)
//		MSSP:	-
//		EUSART:	-
//		ADC:	-
//		RTCC:	Use
//		HLVD:	Vdd monitor
//
//	Environment:
//		IDE: MPLAB IDE 8.36
//      C Compiler for PIC18 MCUs v3.33
//      Microchip Application Libraries v2010-10-19
//      Standard USB bootloader
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2010.12.31 v0.0  First cording
//    2011.01.16 v0.1  12h/24h, fine adjust, auto dimmer and save setting
//    2011.01.17 v0.1  Add on serial number "#DC01999"
//    2011.01.18 v0.1  Bug fix, do not work echo command
//    2011.01.20 v0.1  Response code "ok/ng 999999"
//    2011.01.25 v0.2  PIC18F2xJ50 version
//    2011.01.26 v0.2  Software counter -> RTCC
//    2011.01.31 v0.2  RTCC battery buckup, HLVDN+normal sleep
//    2011.02.01 v0.2  More long buckup, HLVD+Deep sleep+INT0
//    2011.02.02 v0.2  Set time +second from USB
//    2011.02.02 v0.2  Auto dimmer & 12H/24H setting by push switch
//    2011.02.02 v0.2  Fade effect
//    2011.02.03 v0.2  Delete 12h display mode
//    2011.02.03 v0.2  Append calendar function
//    2011.02.05 v0.2  Set dimmer level from USB
//    2011.02.08 v0.2  Bug fix, USB device resume
//    2011.02.08 v0.2  Bug fix, usb_descriptors
//    2011.02.08 v0.2  USB suspend resume dimmer
//    2011.02.14 v0.2a Correct USB suspend
//    2011.02.15 v0.2a Button click sound
//

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "p18cxxx.h"					// Microchip C18 library
#include "timers.h"						// Microchip C18 library
#include "rtcc.h"						// Microchip C18 library
#include "dpslp.h"						// Microchip C18 library
#include "portb.h"						// Microchip C18 library

#include "HardwareProfile.h"			// Microchip Application Libraries v2010-10-19
#include "GenericTypeDefs.h"			// Microchip Application Libraries v2010-10-19
#include "Compiler.h"					// Microchip Application Libraries v2010-10-19
#include "usb_config.h"					// Microchip Application Libraries v2010-10-19
#include "USB/usb.h"					// Microchip Application Libraries v2010-10-19
#include "USB/usb_device.h"				// Microchip Application Libraries v2010-10-19
#include "USB/usb_function_cdc.h"		// Microchip Application Libraries v2010-10-19

#include "led7seg.h"					// Private library
#include "itoanm.h"						// Private library


//
// Application constant
//

// Hardware
#define SW_MODE		(!PORTAbits.RA0)	// Push switch
#define SW_UP		(!PORTAbits.RA1)	// Push switch
#define SW_DOWN		(!PORTAbits.RA2)	// Push switch
#define PUSH_SHORT_TH	100				// Anti chattering
#define PUSH_LONG_TH	3000			// 
#define PUSH_LONG_RP	0xff			// Repeat rate
#define BEEP_INIT()		(TRISAbits.TRISA5 = 0)				// ADD 2011.02.15
#define BEEP_REV()		(LATAbits.LATA5 = ~LATAbits.LATA5)	// ADD 2011.02.15

// Optional function
//#define BLINK_COLON	1				// Second flash
#define BLINK_COLON		2				// 2 Second flash

// Initial value
#define DIM_HI			0				// High dimmer button level
#define DIM_LO			-4				// Low dimmer button level
#define DIM_HI_HH		9				// Dimmer bright start hour
#define DIM_LO_HH		17				// Dimmer dark start hour

// Restrict value
#define TMR1VAL			-7500			// Interval = 1600Hz < CLK(48MHz) / 4 / PS(1) >
#define DIM_MAX			0				// LED Dimmer
#define DIM_MIN			-8				// LED Dimmer

// Inline function
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define ABS(a)		(((a) >  0 ) ? (a) : -(a))

//
// Configuration bits
//
// PIC18F46J50 Family  (24/25/26/44/45/46)
     #pragma config WDTEN = OFF          //WDT disabled (enabled by SWDTEN bit)
     #pragma config PLLDIV = 2           //Divide by 2 (8 MHz internal oscillator input)
     #pragma config STVREN = ON          //stack overflow/underflow reset enabled
     #pragma config XINST = OFF          //Extended instruction set disabled
     #pragma config CPUDIV = OSC1        //No CPU system clock divide -> 48MHz
     #pragma config CP0 = OFF            //Program memory is not code-protected
     #pragma config OSC = INTOSCPLL      //Internal oscillator, PLL enabled, used by USB
     #pragma config T1DIG = ON           //Sec Osc clock source may be selected
     #pragma config LPT1OSC = OFF        //high power Timer1 mode
     #pragma config FCMEN = OFF          //Fail-Safe Clock Monitor disabled
     #pragma config IESO = OFF           //Two-Speed Start-up disabled
     #pragma config WDTPS = 32768        //1:32768
     #pragma config DSWDTOSC = INTOSCREF //DSWDT uses INTOSC/INTRC as clock
     #pragma config RTCOSC = T1OSCREF    //RTCC uses T1OSC/T1CKI as clock
     #pragma config DSBOREN = OFF        //Zero-Power BOR disabled in Deep Sleep
     #pragma config DSWDTEN = OFF        //Disabled
     #pragma config DSWDTPS = 8192       //1:8,192 (8.5 seconds)
     #pragma config IOL1WAY = OFF        //IOLOCK bit can be set and cleared
     #pragma config MSSP7B_EN = MSK7     //7 Bit address masking
     #pragma config WPFP = PAGE_1        //Write Protect Program Flash Page 0
     #pragma config WPEND = PAGE_0       //Start protection at page 0
     #pragma config WPCFG = OFF          //Write/Erase last page protect Disabled
     #pragma config WPDIS = OFF          //WPFP[5:0], WPEND, and WPCFG bits ignored  


//********************************************************************
//
// Global area
//
//********************************************************************

//
// Tables
//
#define MODE_INIT			0
#define MODE_NORMAL			1
#define MODE_SEC			2
#define MODE_DATE			3
#define MODE_ADJ_YY			4
#define MODE_ADJ_MM			5
#define MODE_ADJ_DD			6
#define MODE_ADJ_HH			7
#define MODE_ADJ_MI			8
#define MODE_HOST_N			9
#define MODE_SIZE			10
#define EVENT_MODE_PUSH		0
#define EVENT_MODE_LONG		1
#define EVENT_UP_PUSH		2
#define EVENT_UP_LONG		3
#define EVENT_DOWN_PUSH		4
#define EVENT_DOWN_LONG		5
#define EVENT_SIZE			6
#define EVENT_NONE			99
#define ACTION_NOP			0
#define ACTION_HH_UP		1
#define ACTION_HH_DOWN		2
#define ACTION_MI_UP		3
#define ACTION_MI_DOWN		4
#define ACTION_SS_ZERO		5
#define ACTION_DIM_HI		6
#define ACTION_DIM_LO		7
#define ACTION_DIM_SET_HI	8
#define ACTION_DIM_SET_LO	9
#define ACTION_YY_UP		10
#define ACTION_YY_DOWN		11
#define ACTION_MM_UP		12
#define ACTION_MM_DOWN		13
#define ACTION_DD_UP		14
#define ACTION_DD_DOWN		15
#define ACTION_SET_DATE		16
const rom struct {
	unsigned char	next;
	unsigned char	action;
} decision_table[MODE_SIZE][EVENT_SIZE] = {
	//                   EVENT_MODE_PUSH               EVENT_MODE_LONG             EVENT_UP_PUSH                EVENT_UP_LONG                    EVENT_DOWN_PUSH               EVENT_DOWN_LONG 
	/* MODE_INIT   */ { {MODE_ADJ_HH,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP},   {MODE_INIT  ,ACTION_DIM_HI}, {MODE_INIT  ,ACTION_DIM_SET_HI}, {MODE_INIT  ,ACTION_DIM_LO},  {MODE_INIT  ,ACTION_DIM_SET_LO} },
	/* MODE_NORMAL */ { {MODE_SEC   ,ACTION_NOP},     {MODE_ADJ_HH,ACTION_NOP},   {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_SET_HI}, {MODE_NORMAL,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_DIM_SET_LO} },
	/* MODE_SEC    */ { {MODE_DATE  ,ACTION_NOP},     {MODE_SEC   ,ACTION_NOP},   {MODE_SEC   ,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_NOP},        {MODE_SEC   ,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_NOP} },
	/* MODE_DATE   */ { {MODE_NORMAL,ACTION_NOP},     {MODE_ADJ_YY,ACTION_NOP},   {MODE_DATE  ,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_NOP},        {MODE_DATE  ,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_NOP} },
	/* MODE_ADJ_YY */ { {MODE_ADJ_MM,ACTION_NOP},     {MODE_DATE  ,ACTION_NOP},   {MODE_ADJ_YY,ACTION_YY_UP},  {MODE_ADJ_YY,ACTION_YY_UP},      {MODE_ADJ_YY,ACTION_YY_DOWN}, {MODE_ADJ_YY,ACTION_YY_DOWN}},
	/* MODE_ADJ_MM */ { {MODE_ADJ_DD,ACTION_NOP},     {MODE_DATE  ,ACTION_NOP},   {MODE_ADJ_MM,ACTION_MM_UP},  {MODE_ADJ_MM,ACTION_MM_UP},      {MODE_ADJ_MM,ACTION_MM_DOWN}, {MODE_ADJ_MM,ACTION_MM_DOWN}},
	/* MODE_ADJ_DD */ { {MODE_DATE,  ACTION_SET_DATE},{MODE_DATE  ,ACTION_NOP},   {MODE_ADJ_DD,ACTION_DD_UP},  {MODE_ADJ_DD,ACTION_DD_UP},      {MODE_ADJ_DD,ACTION_DD_DOWN}, {MODE_ADJ_DD,ACTION_DD_DOWN}},
	/* MODE_ADJ_HH */ { {MODE_ADJ_MI,ACTION_NOP},     {MODE_NORMAL,ACTION_NOP},   {MODE_ADJ_HH,ACTION_HH_UP},  {MODE_ADJ_HH,ACTION_HH_UP},      {MODE_ADJ_HH,ACTION_HH_DOWN}, {MODE_ADJ_HH,ACTION_HH_DOWN}},
	/* MODE_ADJ_MI */ { {MODE_NORMAL,ACTION_SS_ZERO}, {MODE_NORMAL,ACTION_NOP},   {MODE_ADJ_MI,ACTION_MI_UP},  {MODE_ADJ_MI,ACTION_MI_UP},      {MODE_ADJ_MI,ACTION_MI_DOWN}, {MODE_ADJ_MI,ACTION_MI_DOWN}},
	/* MODE_HOST_N */ { {MODE_NORMAL,ACTION_NOP},     {MODE_NORMAL,ACTION_NOP},   {MODE_NORMAL,ACTION_DIM_HI}, {MODE_NORMAL,ACTION_DIM_SET_HI}, {MODE_NORMAL,ACTION_DIM_LO},  {MODE_NORMAL,ACTION_DIM_SET_LO} }
};

//
// Inter function variables
//


//********************************************************************
//
// Interrupt Service Routine
//
//********************************************************************
#pragma code
#pragma interrupt HighPriorityISRCode
void HighPriorityISRCode() {
	if (PIR2bits.LVDIF) {
//		PIR2bits.LVDIF = 0;
		PIE2bits.LVDIE = 0;
	} else if (PIR2bits.USBIF) {
		USBDeviceTasks();
		PIR2bits.USBIF = 0;
	}
}
#pragma interruptlow LowPriorityISRCode
void LowPriorityISRCode() {
	if (PIE1bits.TMR1IE && PIR1bits.TMR1IF) {
		WriteTimer1(TMR1VAL);			// House keeping

		// LED scan
		led7seg_Refresh();

		PIR1bits.TMR1IF = 0;			//
	}
}

//********************************************************************
//
// Vector address
//
//********************************************************************
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
void _reset (void){
	    _asm goto _startup _endasm
}
#endif
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void) {
     _asm goto HighPriorityISRCode _endasm
}
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void) {
     _asm goto LowPriorityISRCode _endasm
}
#pragma code

//********************************************************************
//
// USB Device support (Microchip library)
//
//********************************************************************
// Callbacks
void USBCBSuspend(void) {
	// Reduction power			// ADD 2011.02.14
	OSCCONbits.SCS = 0b01;		// ADD 2011.02.14 SEC_IDLE mode
	led7seg_Stop();				// ADD 2011.02.14
	PIE1bits.TMR1IE = 0;		// ADD 2011.02.14
}

void USBCBWakeFromSuspend(void) {
	// Resume normal power		// ADD 2011.02.14
	led7seg_Start();			// ADD 2011.02.14
	PIE1bits.TMR1IE = 1;		// ADD 2011.02.14
	OSCCONbits.SCS = 0b00;		// ADD 2011.02.14 PRI_IDLE mode
}

void USBCB_SOF_Handler(void) {
}

void USBCBErrorHandler(void) {
}

void USBCBStdSetDscHandler(void) {
}

void USBCBCheckOtherReq(void) {
    USBCheckCDCRequest();
}

void USBCBInitEP(void) {
    CDCInitEP();
}

#ifdef DELETE_CODE
void USBCBSendResume(void) {
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do {
        delay_count--;
    } while(delay_count);
    USBResumeControl = 0;
}
#endif 

#if defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived(void) {
}
#endif

BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size) {
    switch(event) {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

//********************************************************************
//
// Subroutines
//
//********************************************************************
void usb_puts(char *buf) {

	if (!USBIsDeviceSuspended()) {
		if(USB_BUS_SENSE && (USBGetDeviceState() == DETACHED_STATE)){
			USBDeviceAttach();
		}
		if (USBGetDeviceState() >= CONFIGURED_STATE) {
			int i;

			for(i = 300; i > 0; --i) {
				if(mUSBUSARTIsTxTrfReady()) break;
				CDCTxService();
			}
			if (i > 0) {
 				putsUSBUSART(buf);
				CDCTxService();
			}
		}
	}
	return;
}

char *usb_gets(char *buf, int sz) {

	if (!USBIsDeviceSuspended()) {
		if(USB_BUS_SENSE && (USBGetDeviceState() == DETACHED_STATE)){
			USBDeviceAttach();
		}
		if (USBGetDeviceState() >= CONFIGURED_STATE) {
			static BYTE		p = 0;

			--sz;
			p += getsUSBUSART(&buf[p], sz-p);	// Aveilable an echo command
			if ((p >= sz)          ||
			    (buf[p-1] == '\r') ||
	    		(buf[p-1] == '\n')) {
				if (p < sz) {
					buf[p] = '\0';
				}
				p = 0;
				return(buf);
			}
		}
	}
	return(NULL);
}

unsigned char bcd2bin(unsigned char bcd) {
	return((bcd >> 4) * 10 + (bcd & 0x0f));
}

unsigned char bin2bcd(unsigned char bin) {
	return(((bin / 10) << 4) | (bin % 10));
}

void click_snd(void){					// ADD 2011.02.15
	unsigned char	i;					// ADD 2011.02.15
										// ADD 2011.02.15
	for(i = 200; i > 0; --i) {			// ADD 2011.02.15
		int j;							// ADD 2011.02.15
										// ADD 2011.02.15
		BEEP_REV();						// ADD 2011.02.15
		for(j = 40; j > 0; --j);		// ADD 2011.02.15
	}									// ADD 2011.02.15
}										// ADD 2011.02.15


//********************************************************************
//
//		M A I N
//
//********************************************************************
void main(void){
	unsigned char	mode;				// State
	short			key_cnt_mode;		// Push count
	short			key_cnt_up;			//
	short			key_cnt_down;		//
	short			host_n;				// Host display mode
	unsigned char	host_dp;			// Dot point column
	short			i;					// etc
	unsigned char	hh_s;				// Hour glitch
//	unsigned char	sp_s;				// USB suspend glitch  DEL 2011.02.14
	signed char		dim_level;			//
	signed char		dim_now;			//
	unsigned char	dim_hi_hh;			//
	unsigned char	dim_lo_hh;			//

	//
	// init
	//
	// Wake from deep sleep
	if(IsResetFromDeepSleep()) {
		ReleaseDeepSleep();				// Release the Deep sleep (IO configuration)
		mode = MODE_NORMAL;

	// Power on initial boot
	} else {
		rtccDate	d;
		rtccTime	t;

		// Setup RTCC
		RtccInitClock();
		RtccWrOn();
		mRtccOn();
		RtccSetCalibration(0);
		d.f.year = 0x11;				// BCD
		d.f.mon  = 0x01;
		d.f.mday = 0x01;
		d.f.wday = 0x00;
		RtccWriteDate(&d, TRUE);
		t.l = 0;
		RtccWriteTime(&t, TRUE);
		mode = MODE_INIT;
	}
	T1CONbits.T1OSCEN = 1;

	// General
//	WDTCONbits.SWDTEN  = 1;				// WDT start for about 1sec sleep
	OSCCONbits.IDLEN   = 1;				// No stop peripherals clock on SLEEP
	INTCONbits.GIE     = 1;
	INTCONbits.PEIE    = 1;
	RCONbits.IPEN      = 1;				// Priority mechanism
//	ANCON0             = 0b00000111;	// Disable AN0~2
	ANCON0             = 0b11111111;	// Disable Analog
	ANCON1             = 0b11111111;	//

	// 7-Seg LED
	IPR1bits.TMR1IP = 0;				// Low priority intrrupt
	led7seg_Start();
	OpenTimer1(TIMER_INT_ON & T1_16BIT_RW & T1_SOURCE_FOSC & T1_PS_1_1 & T1_OSC1EN_ON & T1_SYNC_EXT_OFF,
	           TIMER_GATE_OFF);			// Auto refresh interval

	// USB module
    {
        unsigned int pll_startup_counter;

		IPR2bits.USBIP = 1;				// High priority intrrupt
        pll_startup_counter = 600;
        OSCTUNEbits.PLLEN = 1;			//Enable the PLL and wait 2+ms until the PLL locks before enabling USB module
        while(pll_startup_counter--);
		USBDeviceInit();
		USBDeviceAttach();
    }

	// High/Low voltage detect module
	IPR2bits.LVDIP = 1;					// High priority irq
//	HLVDCONbits.HLVDL = 0b1011;			// 2.90V
//	HLVDCONbits.HLVDL = 0b1100;			// 3.00V
	HLVDCONbits.HLVDL = 0b1101;			// 3.13V
	HLVDCONbits.VDIRMAG = 0;			// Fall edge
	HLVDCONbits.HLVDEN = 1;
    while(HLVDCONbits.BGVST == 0);      // wait for stable bandgap reference voltage
    while(HLVDCONbits.IRVST == 0);      // wait for stable HLVD internal reference voltage
	PIR2bits.LVDIF = 0;
//	PIE2bits.LVDIE = 1;

	// Beep
	BEEP_INIT();

	// Application variables
	key_cnt_mode = 0;
	key_cnt_up   = 0;
	key_cnt_down = 0;
	hh_s         = 24;
//	sp_s         = 1;					// DEL 2011.02.14
	dim_level    = DIM_MAX;
	dim_now      = DIM_MAX;
	dim_hi_hh    = DIM_HI_HH;
	dim_lo_hh    = DIM_LO_HH;


	//
	// Main loop
	//
	for(i = 0; ; i++) {
		unsigned char	e;
		char			cmd[30];
		unsigned char	yy;
		unsigned char	mm;
		unsigned char	dd;
		unsigned char	hh;
		unsigned char	mi;
		unsigned char	ss;
		signed char		dim_save;
		rtccTime		t;
		rtccDate		d;

		//
		// Check event
		//
		e = EVENT_NONE;

		// Get RTCC
		if (mode < MODE_ADJ_YY) {
			RtccReadDate(&d);
			yy = bcd2bin(d.f.year);
			mm = bcd2bin(d.f.mon);
			dd = bcd2bin(d.f.mday);
			RtccReadTime(&t);
			hh = bcd2bin(t.f.hour);
			mi = bcd2bin(t.f.min);
			ss = bcd2bin(t.f.sec);
		}

		// Push keys
		if (SW_MODE) {
			key_cnt_mode++;
			if (key_cnt_mode == PUSH_LONG_TH) {
				e = EVENT_MODE_LONG;
			}
		} else {
			if ((key_cnt_mode > PUSH_SHORT_TH) &&
			    (key_cnt_mode < PUSH_LONG_TH)) {
				e = EVENT_MODE_PUSH;
			}
			key_cnt_mode = 0;
		}
		if (SW_UP) {
			key_cnt_up++;
			if (key_cnt_up > PUSH_LONG_TH) {
				if ((key_cnt_up & PUSH_LONG_RP) == 0) {
					e = EVENT_UP_LONG;
				}
			} else if (key_cnt_up == PUSH_SHORT_TH) {
				e = EVENT_UP_PUSH;
			}
		} else {
			key_cnt_up = 0;
		}
		if (SW_DOWN) {
			key_cnt_down++;
			if (key_cnt_down > PUSH_LONG_TH) {
				if ((key_cnt_down & PUSH_LONG_RP) == 0) {
					e = EVENT_DOWN_LONG;
				}
			} else if (key_cnt_down == PUSH_SHORT_TH) {
				e = EVENT_DOWN_PUSH;
			}
		} else {
			key_cnt_down = 0;
		}
		if (e != EVENT_NONE) {			// ADD 2011.02.15
			click_snd();				// ADD 2011.02.15
		}								// ADD 2011.02.15

		//
		// Action
		//
		if (e < EVENT_SIZE) {

			switch(decision_table[mode][e].action) {
			case ACTION_HH_UP:
				hh++;
				if (hh >= 24) {
					hh = 0;
				}
				break;
			case ACTION_HH_DOWN:
				if (hh == 0) {
					hh = 24;
				}
				--hh;
				break;
			case ACTION_MI_UP:
				mi++;
				if (mi >= 60) {
					mi = 0;
				}
				break;
			case ACTION_MI_DOWN:
				if (mi == 0) {
					mi = 60;
				}
				--mi;
				break;
			case ACTION_SS_ZERO:
				ss = 0;
				t.f.hour = bin2bcd(hh);
				t.f.min  = bin2bcd(mi);
				t.f.sec  = bin2bcd(ss);
				RtccWriteTime(&t, TRUE);
				break;
			case ACTION_DIM_HI:
				dim_level = DIM_HI;
				break;
			case ACTION_DIM_LO:
				dim_level = DIM_LO;
				break;
			case ACTION_DIM_SET_HI:
				dim_hi_hh = hh;
				break;
			case ACTION_DIM_SET_LO:
				dim_lo_hh = hh;
				break;
			case ACTION_YY_UP:
				yy++;
				if (yy > 99) {
					yy = 0;
				}
				break;
			case ACTION_YY_DOWN:
				if (yy == 0) {
					yy = 100;
				}
				--yy;
				break;
			case ACTION_MM_UP:
				mm++;
				if (mm > 12) {
					mm = 1;
				}
				break;
			case ACTION_MM_DOWN:
				--mm;
				if (mm == 0) {
					mm = 12;
				}
				break;
			case ACTION_DD_UP:
				dd++;
				if (dd > 31) {
					dd = 1;
				}
				break;
			case ACTION_DD_DOWN:
				--dd;
				if (dd == 0) {
					dd = 31;
				}
				break;
			case ACTION_SET_DATE:
				d.f.year = bin2bcd(yy);
				d.f.mon  = bin2bcd(mm);
				d.f.mday = bin2bcd(dd);
				d.f.wday = 0;
				RtccWriteDate(&d, FALSE);
				break;
			default:
				break;
			}
	
			mode = decision_table[mode][e].next;
		}

#ifdef DELETE_CODE						// DEL start 2011.02.14
		// Suspend dimmer
		if (sp_s != USBIsDeviceSuspended()) {
			sp_s = USBIsDeviceSuspended();
			if (sp_s == 0) {
				dim_level = DIM_MAX;
			} else {
				dim_level = DIM_MIN;
			}
		}
#endif									// DEL end 2011.02.14

		//
		// Display
		//
		switch(mode) {
		case MODE_SEC:
			led7seg_DispInt(mi    , 1, 2);
			led7seg_DispInt(ss+100, 3, 2);
			led7seg_DP(1, 2);
			break;
		case MODE_DATE:
			led7seg_DispInt(mm, 1, 2);
			led7seg_DispInt(dd, 3, 2);
			led7seg_DP(0, 2);
			break;
		case MODE_HOST_N:
			led7seg_DispInt(host_n, 1, 4);
			led7seg_DP(0, 0);
			if (host_dp != 4) {
				led7seg_DP(1, host_dp);
			}
			break;
		case MODE_ADJ_YY:
			if (i & 0x200) {
				led7seg_PutPattern(0x00, 0);
			} else {
				led7seg_DispInt(yy+2000, 1, 4);
			}
			break;
		case MODE_ADJ_MM:
			if (i & 0x200) {
				led7seg_PutPattern(0x00, 1);
				led7seg_PutPattern(0x00, 2);
			} else {
				led7seg_DispInt(mm, 1, 2);
				led7seg_DispInt(dd, 3, 2);
			}
			break;
		case MODE_ADJ_DD:
			if (i & 0x200) {
				led7seg_PutPattern(0x00, 3);
				led7seg_PutPattern(0x00, 4);
			} else {
				led7seg_DispInt(mm, 1, 2);
				led7seg_DispInt(dd, 3, 2);
			}
			break;
		default:
//		case MODE_INIT:
//		case MODE_NORMAL:
//		case MODE_ADJ_HH:
//		case MODE_ADJ_MI:
			if ((mode == MODE_INIT) && (ss & 0x01)) {
				led7seg_PutPattern(0x00, 0);
			} else if ((mode == MODE_ADJ_HH) && (i & 0x100)) {
				led7seg_PutPattern(0x00, 1);
				led7seg_PutPattern(0x00, 2);
			} else if ((mode == MODE_ADJ_MI) && (i & 0x100)) {
				led7seg_PutPattern(0x00, 3);
				led7seg_PutPattern(0x00, 4);
			} else {
				led7seg_DispInt(hh, 1, 2);
				led7seg_DispInt(mi+100, 3, 2);
				switch(ss) {
				case 59:
					dim_level = DIM_MIN;
					break;
				case 0:
					dim_level = dim_save;
					break;
				default:
					dim_save = dim_level;
					break;
				}
			}
#if BLINK_COLON == 1
			if (mRtccIs2ndHalfSecond()) {
				led7seg_DP(1, 2);
			} else {
				led7seg_DP(0, 2);
			}
#elif BLINK_COLON == 2
			if (ss & 0x01) {
				led7seg_DP(1, 2);
			} else {
				led7seg_DP(0, 2);
			}
#else
			led7seg_DP(1, 2);
#endif
				
			break;
		}

		// Dimmer control
		if (hh_s != hh) {				// Timer on/off
			hh_s = hh;
			if (hh == dim_hi_hh) {
				dim_level = DIM_HI;
			}
			if (hh == dim_lo_hh) {
				dim_level = DIM_LO;
			}
		}
		if ((i & 0x7f) == 0) {			// Smooth dimming
			if (dim_now > dim_level) {
				--dim_now;
			}
			if (dim_now < dim_level) {
				dim_now++;
			}
			led7seg_Dim((dim_now == 0) ? 1 : dim_now);
		}

		//
		// USB Command
		//
   	    if (usb_gets(cmd, sizeof(cmd)-3) != NULL) {
			char			*p;			// Work
			short			v;			// Parameter value
			unsigned char	v1;			//
			unsigned char	v2;			//
			unsigned char	v3;			//
			unsigned char	dp;			// Dot point column
			unsigned char	f;			// ok/ng flag
			unsigned short	r;			// Response code

			p = strchr(cmd, '\0');
			*(p-1) = '\r';
			*p++   = '\n';
			*p     = '\0';
//			strtokpgmram(cmd, "\r\n");
//			strcatpgm2ram(cmd, "\r\n");
			usb_puts(cmd);				// Echo back

			dp = 0;						// Argument cut in dot
			p = strchr(cmd, '.');
			if (p != NULL) {
				for(; *p != '\0'; p++) {
					*p = *(p+1);
					if (isdigit(*p)) {
						dp++;
					}
				}
			}
			v = atoi(&cmd[2]);
			v3 = bin2bcd(atoi(&cmd[6]));
			cmd[6] = '\0';
			v2 = bin2bcd(atoi(&cmd[4]));
			cmd[4] = '\0';
			v1 = bin2bcd(atoi(&cmd[2]));

			r = 0;
			f = 1;
			if ((cmd[0] == 's') && (cmd[1] == 'd')) {
				d.f.year = v1;
				d.f.mon  = v2;
				d.f.mday = v3;
				d.f.wday = 0;
				RtccWriteDate(&d, FALSE);
				mode = MODE_DATE;
			} else if ((cmd[0] == 's') && (cmd[1] == 't')) {
				t.f.hour = v1;
				t.f.min  = v2;
				t.f.sec  = v3;
				RtccWriteTime(&t, TRUE);
				mode = MODE_NORMAL;
			} else if ((cmd[0] == 'n') && (cmd[1] == 'm')) {
				host_n  = v;
				host_dp = 4 - dp;
				mode = MODE_HOST_N;
			} else if ((cmd[0] == 't') && (cmd[1] == 'm')) {
				mode = MODE_NORMAL;
			} else if ((cmd[0] == 'a') && (cmd[1] == 'd')) {
				signed char	c;

				c = RTCCAL;
				c += v;
				r = c;
				RTCCAL = c;
			} else if ((cmd[0] == 'd') && (cmd[1] == 'h')) {
				if (isdigit(cmd[2])) {
					dim_hi_hh = v;
				} else {
					r = dim_hi_hh;
				}
			} else if ((cmd[0] == 'd') && (cmd[1] == 'l')) {
				if (isdigit(cmd[2])) {
					dim_lo_hh = v;
				} else {
					r = dim_lo_hh;
				}
			} else if ((cmd[0] == 'd') && (cmd[1] == 'm')) {
				if (isdigit(cmd[2])) {
					dim_level = v - 8;
				} else {
					r = v - dim_level;
				}
			} else if (cmd[0] == '?') {
				const rom char *msg[] = {
					"sd991231: Set date\r\n",
					"st235959: Set time\r\n",
					"nm9999: Num mode\r\n",
					"tm: Time mode\r\n",
					"ad-99: Calibrate\r\n",
					"dh99: Dimmer high\r\n",
					"dl99: Dimmer low\r\n",
					"dm9: Dimmer set\r\n",
					NULL
				};
				unsigned char	n;

				for(n = 0; ; n++) {
					if (msg[n] == NULL) break;
					strcpypgm2ram(cmd, msg[n]);
					usb_puts(cmd);
				}
			} else {
				f = 0;
				r = 1;
			}

			// Send response
			if (f) {
				cmd[0] = 'o';
				cmd[1] = 'k';
			} else {
				cmd[0] = 'n';
				cmd[1] = 'g';
			}
			itoanm(r, &cmd[2] , 6, -1);
			cmd[8] = '\r';
			cmd[9] = '\n';
			cmd[10] = '\0';
			usb_puts(cmd);
		}

		//
		// Power-down detect
		//
		if (PIR2bits.LVDIF) {
			int	i;

			// Verify
			PIR2bits.LVDIF = 0;
			for(i = 100; i > 0; --i) {
				if (PIR2bits.LVDIF) {
					break;
				}
			}
			if (i > 0) {

				// Reduction power
//				OSCCONbits.SCS = 0b01;		// SEC_RUN mode, a little time
				CloseTimer1();
				led7seg_Stop();
				USBDeviceDetach();

				// Sleep until RB0 signal
				OpenRB0INT(PORTB_CHANGE_INT_ON | RISING_EDGE_INT | PORTB_INT_PRIO_HIGH | PORTB_PULLUPS_OFF);
				GotoDeepSleep(DPSLP_ULPWU_ENABLE | DPSLP_RTCC_WAKEUP_DISABLE);
				Reset();					// Dummy instruction
			}
		}

		//
		// Waiting for event
		//
		Sleep();						// Keep peripherals clock
		Nop();
		ClrWdt();
	}
}
